SystemOrganization addCategory: #'Mapless-Mongo'!
SystemOrganization addCategory: #'Mapless-Models'!
SystemOrganization addCategory: #'Mapless-Core'!
SystemOrganization addCategory: #'Mapless-Mongo-Tests'!
SystemOrganization addCategory: #'Mapless-Mongo-Errors'!
SystemOrganization addCategory: #'Mapless-Errors'!

Exception subclass: #MaplessError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Errors'!

MaplessError subclass: #MaplessModelGetterIsMissing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Errors'!

MaplessError subclass: #MaplessModelMetadataError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Errors'!

Mongo subclass: #MongoAPI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo'!

!MongoAPI methodsFor: 'testing' stamp: 'sebastiansastre 12/2/2012 17:07'!
isValid 

	^ self isOpen ! !

!MongoAPI methodsFor: 'actions' stamp: 'sebastiansastre 12/2/2012 17:14'!
reset 
	"Close, no fail"
	[ self close ] on: Error do: [ :x |  ] ! !

DynamicVariable subclass: #MongoCurrentClient
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo'!

FlowError subclass: #MaplessMongoError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo-Errors'!

MaplessMongoError subclass: #MaplessMongoDatabaseDowntime
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo-Errors'!

MaplessMongoError subclass: #MaplessMongoUnavailableDatabase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo-Errors'!

MaplessMongoError subclass: #MaplessUnsavedMongoSubmodel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo-Errors'!

TestCase subclass: #MaplessMongoTest
	instanceVariableNames: 'pool'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo-Tests'!

!MaplessMongoTest class methodsFor: 'history' stamp: 'sebastiansastre 12/3/2012 18:19'!
lastStoredRun
	^ ((Dictionary new) add: (#passed->((Set new) add: #testDefaultCategoryCleanUp; add: #testPackageCleanUp; add: #testSingleClassCreation; add: #testClassCreationInDifferentCategories; add: #testClassFastCreationInDifferentCategories; add: #testMultipleClassCreation; add: #testSingleClassFastCreation; yourself)); add: (#timeStamp->'22 November 2008 10:11:35 pm'); add: (#failures->((Set new))); add: (#errors->((Set new))); yourself)! !

!MaplessMongoTest methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 22:32'!
do: aBlock 

	^ pool databaseAt: 'flow' do: aBlock ! !

!MaplessMongoTest methodsFor: 'setUp-tearDown' stamp: 'sebastiansastre 12/3/2012 18:21'!
setUp
	super setUp.
	pool := MongoPool instance.! !

!MaplessMongoTest methodsFor: 'setUp-tearDown' stamp: 'sebastiansastre 12/4/2012 19:02'!
tearDown
	super tearDown.

	pool reset.
	pool := nil! !

!MaplessMongoTest methodsFor: 'testing' stamp: 'sebastiansastre 1/4/2013 09:05'!
testComposedAccess
	"Test proxies and its on-demand-load and DNU mechanism"

	| user guy |
	
	user := MaplessDummyUser new
				userame: 'johnq';
				yourself.
	
	guy := MaplessDummyPerson new
			firstName: 'john';
			lastName: 'q';
			yourself.

	user person: guy.

	self do:[ 
		guy save.
		user save ].
	
	self do:[
		self assert: ( MaplessDummyUser findAt: user oid ) completeName = 'john q'.
		self assert: ( MaplessDummyUser findAt: user oid ) person class = MaplessMongoReference.
		self assert: ( MaplessDummyUser findAt: user oid ) person model class = MaplessDummyPerson.
		self assert: ( MaplessDummyUser findAt: user oid ) person firstName = 'john'.
		self assert: ( MaplessDummyUser findAt: user oid ) person lastName = 'q' ]
! !

!MaplessMongoTest methodsFor: 'testing' stamp: 'sebastiansastre 1/4/2013 09:04'!
testComposedDelete

	| users |
	
	users := OrderedCollection new.

	Character alphabet do: [ :char |
		users add: (MaplessDummyUser new 
					userame: char asString;
					person: (MaplessDummyPerson new
								firstName: char asString;
								yourself);
					yourself)].
	
	self assert: ( users allSatisfy: [ :user | user isUnsaved ] ).	
	self assert: ( users allSatisfy: [ :user | user person isUnsaved ] ).	

	self do:[ users do:[ :user | user person save ] ].
	self do:[ users do:[ :user | user save ] ].

	self assert: ( users allSatisfy: [ :user | user isUnsaved not ] ).	
	self assert: ( users allSatisfy: [ :user | user person isUnsaved not ] ).	
	self assert: ( users allSatisfy: [ :user | user isDeleted not ] ).	
	self assert: ( users allSatisfy: [ :user | user person isDeleted not ] ).	

	self do:[ users do:[ :user | user person delete ] ].
	
	self assert: ( users allSatisfy: [ :user | user person isDeleted ] ).	
	self assert: ( users allSatisfy: [ :user | user isDeleted not ] ).	

	self do:[ users do:[ :user | user delete ] ].

	self assert: ( users allSatisfy: [ :user | user person isDeleted ] ).	
	self assert: ( users allSatisfy: [ :user | user isDeleted ] ).	
! !

!MaplessMongoTest methodsFor: 'testing' stamp: 'sebastiansastre 1/4/2013 09:04'!
testComposedDestroy

	| users |
	
	users := OrderedCollection new.

	Character alphabet do: [ :char |
		users add: (MaplessDummyUser new 
					userame: char asString;
					person: (MaplessDummyPerson new
								firstName: char asString;
								yourself);
					yourself)].
	
	self assert: ( users allSatisfy: [ :user | user isUnsaved ] ).	
	self assert: ( users allSatisfy: [ :user | user person isUnsaved ] ).	

	self do:[ users do:[ :user | user person save ] ].
	self do:[ users do:[ :user | user save ] ].

	self assert: ( users allSatisfy: [ :user | user isUnsaved not ] ).	
	self assert: ( users allSatisfy: [ :user | user person isUnsaved not ] ).	

	self do:[ users do:[ :user | user person destroy ] ].
	
	self do:[
		self assert: ( users allSatisfy: [ :user | ( MaplessDummyPerson existsAt: user person oid ) not] ).	
		self assert: ( users allSatisfy: [ :user | MaplessDummyUser existsAt: user oid ] ) ].	

	self do:[ users do:[ :user | user destroy ] ].

	self assert: ( self do: [ users allSatisfy: [ :user | (MaplessDummyUser existsAt: user oid) not ] ] ).	
! !

!MaplessMongoTest methodsFor: 'testing' stamp: 'sebastiansastre 1/4/2013 09:08'!
testComposedSave

	| user guy |
	
	user := MaplessDummyUser new
				userame: 'johnq';
				yourself.
	
	guy := MaplessDummyPerson new
			firstName: 'john';
			lastName: 'q';
			yourself.

	user person: guy.
		
	"Whithout saving a submodel"
	self assert: user isUnsaved.	
	self assert: guy isUnsaved.	
	self should: [ self do:[ user save ] ] raise: MaplessUnsavedMongoSubmodel.

	"After saving that submodel"
	self do:[ guy save ].
	self shouldnt: [ self do:[ user save ] ] raise: MaplessUnsavedMongoSubmodel.
	
	self deny: user isUnsaved.	
	self deny: guy isUnsaved.	

! !

!MaplessMongoTest methodsFor: 'testing' stamp: 'sebastiansastre 1/4/2013 09:08'!
testComposedSaveAndLoad

	| user guy loaded |
	
	user := MaplessDummyUser new
				userame: 'johnq';
				yourself.
	
	guy := MaplessDummyPerson new
			firstName: 'john';
			lastName: 'q';
			yourself.

	user person: guy.
		
	"Whithout saving a submodel"
	self assert: user isUnsaved.	
	self assert: guy isUnsaved.	
	self should: [ self do:[ user save ] ] raise: MaplessUnsavedMongoSubmodel.

	"After saving that submodel"
	self do:[ guy save ].
	self shouldnt: [ self do:[ user save ] ] raise: MaplessUnsavedMongoSubmodel.
	
	self deny: user isUnsaved.	
	self deny: guy isUnsaved.	

	self do:[ loaded := MaplessDummyUser findAt: user oid ].
	self assert: loaded notNil.
	self deny: loaded isUnsaved.
	self assert: loaded data = user data.

! !

!MaplessMongoTest methodsFor: 'testing' stamp: 'sebastiansastre 1/4/2013 09:05'!
testComposition

	| user guy |
	
	user := MaplessDummyUser new
				userame: 'johnq';
				yourself.
	
	guy := MaplessDummyPerson new
			firstName: 'john';
			lastName: 'q';
			yourself.

	user person: guy.
		
	self assert: guy isUnsaved.	
	self assert: ((user data at: 'person') class = MaplessMongoReference).
	self assert: ((user data at: 'person') behavior = #MaplessDummyPerson).
	self assert: ((user data at: 'person') oid = guy oid).
	self assert: (user completeName = 'john q').
	
	
	! !

!MaplessMongoTest methodsFor: 'testing' stamp: 'sebastiansastre 1/4/2013 09:04'!
testDelete

	| people |
	
	people := OrderedCollection new.

	Character alphabet do: [ :char |
		people add: (MaplessDummyPerson new
						firstName: char asString;
						yourself)].
	
	self assert: ( people allSatisfy: [ :guy | guy isUnsaved ] ).	

	self do:[ people do:[ :guy | guy save ] ].

	self assert: ( people allSatisfy: [ :guy | guy isUnsaved not ] ).	
	self assert: ( people allSatisfy: [ :guy | guy isDeleted not ] ).	
	
	self do:[ people do:[ :guy | guy delete ] ].
	self assert: ( people allSatisfy: [ :guy | guy isDeleted ] ).	
! !

!MaplessMongoTest methodsFor: 'testing' stamp: 'sebastiansastre 1/4/2013 09:04'!
testDestroy

	| people |
	
	people := OrderedCollection new.

	Character alphabet do: [ :char |
		people add: (MaplessDummyPerson new
						firstName: char asString;
						yourself)].
	
	self assert: ( people allSatisfy: [ :guy | guy isUnsaved ] ).	

	self do:[ people do:[ :guy | guy save ] ].

	self assert: ( people allSatisfy: [ :guy | guy isUnsaved not ] ).	
	self assert: ( people allSatisfy: [ :guy | guy isDeleted not ] ).	
	
	self do:[ people do:[ :guy | guy destroy ] ].
	self assert: ( self do:[ people allSatisfy: [ :guy | ( MaplessDummyPerson existsAt: guy oid ) not ] ]).
! !

!MaplessMongoTest methodsFor: 'testing' stamp: 'sebastiansastre 1/4/2013 09:04'!
testQueryUsers

	| users |
	
	users := OrderedCollection new.

	Character alphabet do: [ :char |
		users add: (MaplessDummyUser new 
					username: char asString;
					person: (MaplessDummyPerson new
								firstName: char asString;
								yourself);
					yourself)].
	
	self assert: ( users allSatisfy: [ :user | user isUnsaved ] ).	
	self assert: ( users allSatisfy: [ :user | user person isUnsaved ] ).	

	self do:[ users do:[ :user | user person save ] ].
	self do:[ users do:[ :user | user save ] ].

	self do:[ 
		self assert: (Character alphabet allSatisfy: [ :char | 
						( MaplessDummyUser findUsername: char asString ) notNil ] ) ]
! !

!MaplessMongoTest methodsFor: 'testing' stamp: 'sebastiansastre 1/4/2013 09:04'!
testSimpleSave

	| guy |
	
	guy := MaplessDummyPerson new
			firstName: 'john';
			lastName: 'q';
			yourself.
	
	self assert: guy isUnsaved.	
	self do:[ guy save ].
	self deny: guy isUnsaved.
! !

!MaplessMongoTest methodsFor: 'testing' stamp: 'sebastiansastre 1/4/2013 09:04'!
testSimpleSaveAndLoad
	| guy loaded |
	
	guy := MaplessDummyPerson new
			firstName: 'john';
			lastName: 'q';
			yourself.
	
	self assert: guy isUnsaved.	
	self do:[guy save].
	self deny: guy isUnsaved.

	self do:[ loaded := MaplessDummyPerson findAt: guy oid ].
	self assert: loaded notNil.
	self deny: loaded isUnsaved.
	self assert: loaded data = guy data.
	! !

Object subclass: #MaplessAbstractPool
	instanceVariableNames: 'clientsMutex busyClients idleClients'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Core'!

MaplessAbstractPool class
	instanceVariableNames: 'instance'!

MaplessAbstractPool class
	instanceVariableNames: 'instance'!

!MaplessAbstractPool class methodsFor: 'initialization' stamp: 'sebastiansastre 4/19/2012 15:57'!
initializeInstance

	^ instance := self new! !

!MaplessAbstractPool class methodsFor: 'accessing' stamp: 'sebastiansastre 4/19/2012 15:57'!
instance 

	^ instance ifNil:[self initializeInstance]! !

!MaplessAbstractPool class methodsFor: 'actions' stamp: 'sebastiansastre 5/14/2012 15:51'!
reset 

	instance ifNotNil:[
		instance reset].
		
	instance := nil! !

!MaplessAbstractPool methodsFor: 'accessing' stamp: 'sebastiansastre 12/1/2012 21:40'!
busyClients 
	"Answers the bunch of clients that are currently busy"
	^ busyClients ifNil:[self initializeBusyClients]! !

!MaplessAbstractPool methodsFor: 'accessing' stamp: 'sebastiansastre 12/1/2012 21:40'!
clientsMutex
	"Answers the mutex used to protect clients usage"
	^ clientsMutex ifNil: [self initializeClientsMutex]! !

!MaplessAbstractPool methodsFor: 'accessing' stamp: 'sebastiansastre 12/2/2012 16:52'!
currentClientDynamicVariable
	"Answers the class of the dynamic variable that will
	hold the value of the current client during the evaluation of its usage."
	^ self subclassResponsibility ! !

!MaplessAbstractPool methodsFor: 'actions' stamp: 'sebastiansastre 12/3/2012 18:26'!
do: aBlock
	"Evaluates aBlock using a client"

	^ self withClientDo: aBlock 
! !

!MaplessAbstractPool methodsFor: 'testing' stamp: 'sebastiansastre 12/1/2012 21:51'!
hasBusyClients
	"Answers true if this pool has, at least one, busy client"
	^ self busyClients notEmpty! !

!MaplessAbstractPool methodsFor: 'testing' stamp: 'sebastiansastre 12/1/2012 21:51'!
hasClients
	"Answers true if this pool has at least one client"
	^ self hasIdleClients or:[
	self hasBusyClients]! !

!MaplessAbstractPool methodsFor: 'testing' stamp: 'sebastiansastre 12/1/2012 21:51'!
hasIdleClients
	"Answers true if this pool has, at least one, idle client"
	^ self idleClients notEmpty! !

!MaplessAbstractPool methodsFor: 'accessing' stamp: 'sebastiansastre 12/1/2012 21:41'!
idleClients 
	"Answers the bunch of clients that are currently idle"
	^ idleClients ifNil:[self initializeIdleClients]! !

!MaplessAbstractPool methodsFor: 'initialization' stamp: 'sebastiansastre 12/1/2012 21:40'!
initializeBusyClients

	^ busyClients := IdentitySet new! !

!MaplessAbstractPool methodsFor: 'initialization' stamp: 'sebastiansastre 12/1/2012 21:40'!
initializeClientsMutex

	^ clientsMutex := Semaphore forMutualExclusion! !

!MaplessAbstractPool methodsFor: 'initialization' stamp: 'sebastiansastre 12/1/2012 21:40'!
initializeIdleClients

	^ idleClients := IdentitySet new! !

!MaplessAbstractPool methodsFor: 'actions' stamp: 'sebastiansastre 12/2/2012 16:57'!
makeClient
	"Returns a new client so it can be (re)used in the pool"
	^ self subclassResponsibility ! !

!MaplessAbstractPool methodsFor: 'actions' stamp: 'sebastiansastre 12/2/2012 16:41'!
printOn: aStream

	super printOn: aStream.
	
	aStream 
		nextPutAll: '(',self busyClients size asString,' busy, ',self idleClients size asString, ' idle)'! !

!MaplessAbstractPool methodsFor: 'actions' stamp: 'sebastiansastre 12/2/2012 18:02'!
remove: aClient

	self idleClients remove: aClient ifAbsent:[ nil ].
	self busyClients remove: aClient ifAbsent:[ nil ].
	! !

!MaplessAbstractPool methodsFor: 'actions' stamp: 'sebastiansastre 12/2/2012 16:56'!
requestClient
	"Answers a aClient instance taken from the idle clients.
	It'll create a new one if needed.
	It's a thread safe operation."

	| client |

	self clientsMutex critical: [
		"Transcript cr; show: 'getting odb client p: ', Processor activeProcess hash asString."
		client := self hasIdleClients
				ifTrue: [ self idleClients anyOne ]
				ifFalse: [ self makeClient ].
		client isValid ifFalse:[
			self remove: client.
			client := self makeClient ].
		self idleClients remove: client ifAbsent:[ nil ].
		self busyClients add: client].
	
	^ client! !

!MaplessAbstractPool methodsFor: 'actions' stamp: 'sebastiansastre 12/2/2012 18:02'!
reset 

	self busyClients copy do:[ :e | 
		e reset. 
		self remove: e ].

	self idleClients copy do:[ :e | 
		e reset.
		self remove: e ].! !

!MaplessAbstractPool methodsFor: 'actions' stamp: 'sebastiansastre 12/2/2012 16:55'!
returnClient: aClient
	"Sends aClient to go from the busy ones to the idle ones"

	self clientsMutex critical:[
		self busyClients remove: aClient ifAbsent: [self returningUnusedClient].		
		self idleClients add: aClient ]! !

!MaplessAbstractPool methodsFor: 'actions' stamp: 'sebastiansastre 12/1/2012 21:49'!
returningUnusedClient

	Transcript cr; show: 'Warning: returning a client that is not in among the busy ones'
! !

!MaplessAbstractPool methodsFor: 'actions' stamp: 'sebastiansastre 12/2/2012 16:55'!
withClientDo: aBlock
	"Evaluates aBlock using one of the 
	(at the moment) idle clients as parameter.
	Other code could reach the client in use by accessing
	the XXCurrentClient dynamic variable's value."

	| client |

	client := self requestClient.
		
	^ self currentClientDynamicVariable value: client during:[
		[aBlock value: client] ensure:[
			self returnClient: client]]
! !

MaplessAbstractPool subclass: #MongoPool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo'!

!MongoPool methodsFor: 'accessing' stamp: 'sebastiansastre 12/2/2012 17:04'!
currentClientDynamicVariable
	"Answers the class of the dynamic variable that will
	hold the value of the current client during the evaluation of its usage."
	^ MongoCurrentClient! !

!MongoPool methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 22:08'!
databaseAt: aDatabaseName do: aBlock

	^ self do:[ :client |
		(client databaseNamed: aDatabaseName) do: aBlock ]! !

!MongoPool methodsFor: 'actions' stamp: 'sebastiansastre 12/9/2012 16:53'!
ifUnavailablePurge: aMongoClient
	"Tests aMongoClient and remove it from this pool if invalid."

	(aMongoClient isOpen not or:[
	([aMongoClient databases first collections.
	true] on: Error do: [:x| false]) not ] ) ifTrue:[ self remove:  aMongoClient]! !

!MongoPool methodsFor: 'actions' stamp: 'sebastiansastre 12/2/2012 17:07'!
makeClient
	"Returns a new client so it can be (re)used in the pool"

	^ MongoAPI default 
		open;
		yourself ! !

!MongoPool methodsFor: 'reactions' stamp: 'sebastiansastre 12/2/2012 17:04'!
onLogout: aMongo

	self remove: aMongo ! !

!MongoPool methodsFor: 'actions' stamp: 'sebastiansastre 12/9/2012 15:49'!
purgeUnavailable
	"Removes any client that might be expired or unreacheable."
	
	self busyClients copy do:[ :e | self ifUnavailablePurge: e ].
	self idleClients copy do:[ :e | self ifUnavailablePurge: e ]! !

Object subclass: #MaplessMongoBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo'!

!MaplessMongoBenchmark class methodsFor: 'actions' stamp: 'sebastiansastre 12/5/2012 15:47'!
run

	Transcript open.

	self runPrintingOn: Transcript ! !

!MaplessMongoBenchmark class methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 22:49'!
runPrintingOn: aStream

	aStream cr; nextPutAll: 'Benchmarking...'; cr; flush.

	(Delay forDuration: 1 second) wait. 
	self save1KPeopleOn: aStream.
	aStream flush.
	Smalltalk garbageCollect; garbageCollect.
	
	(Delay forDuration: 1 second) wait. 
	self save10KPeopleOn: aStream.
	aStream flush.
	Smalltalk garbageCollect; garbageCollect.

	(Delay forDuration: 1 second) wait. 
	self save1KUsersOn: aStream.
	aStream flush.
	Smalltalk garbageCollect; garbageCollect.

	(Delay forDuration: 1 second) wait. 
	self save10KUsersOn: aStream.
	aStream flush.	
	Smalltalk garbageCollect; garbageCollect.
! !

!MaplessMongoBenchmark class methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 22:39'!
save10KPeopleOn: aStream 

	self save: 10000 peopleOn: aStream ! !

!MaplessMongoBenchmark class methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 22:38'!
save10KUsersOn: aStream 

	self save: 10000 usersOn: aStream ! !

!MaplessMongoBenchmark class methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 22:37'!
save1KPeopleOn: aStream 

	self save: 1000 peopleOn: aStream ! !

!MaplessMongoBenchmark class methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 22:38'!
save1KUsersOn: aStream 

	self save: 1000 usersOn: aStream ! !

!MaplessMongoBenchmark class methodsFor: 'actions' stamp: 'sebastiansastre 1/4/2013 09:04'!
save: aQuantity peopleOn: aStream 

	| firstNames lastNames delay savesPerSecond |

	firstNames := OrderedCollection new.
	lastNames := OrderedCollection new.
	
	1 to: aQuantity do:[ :i |
		firstNames add: Character alphabet shuffled anyOne asString ].

	1 to: aQuantity do:[ :i |
		lastNames add: Character alphabet shuffled anyOne asString ].

	MongoPool instance databaseAt: 'flow' do:[ 
		delay := Time millisecondsToRun: [
			1 to: aQuantity do:[ :i | 
				 MaplessDummyPerson new
					firstName: (firstNames at: i);
					lastName: (lastNames at: i);
					save]]].
	
	savesPerSecond := ((1000 * aQuantity) / delay asFloat) truncated.
	
	aStream cr;
		nextPutAll: 'Saved ';
		nextPutAll: aQuantity asString;
		nextPutAll: ' instances of FlowDummyPerson in: ';
		nextPutAll: delay milliseconds asString;
		nextPutAll: ' ms ';
		nextPutAll: '(~',savesPerSecond asString,' saves per second)'! !

!MaplessMongoBenchmark class methodsFor: 'actions' stamp: 'sebastiansastre 1/4/2013 09:04'!
save: aQuantity usersOn: aStream 

	| firstNames lastNames users delay savesPerSecond |

	firstNames := OrderedCollection new.
	lastNames := OrderedCollection new.
	users := OrderedCollection new.
	
	1 to: aQuantity do:[ :i |
		firstNames add: Character alphabet shuffled anyOne asString ].

	1 to: aQuantity do:[ :i |
		lastNames add: Character alphabet shuffled anyOne asString ].

	1 to: aQuantity do:[ :i | | guy user |
		guy := MaplessDummyPerson new
				firstName: (firstNames at: i);
				lastName: (lastNames at: i);
				yourself.
		user := MaplessDummyUser new
					username: guy firstName;
					person: guy;
					yourself.
		users add: user].

	MongoPool instance databaseAt: 'flow' do:[ 
		delay := Time millisecondsToRun: [
				users do:[ :user | 
					user person save.
					user save]]].
	
	savesPerSecond := ((1000 * aQuantity) / delay asFloat) truncated.
	
	aStream cr;
		nextPutAll: 'Saved ';
		nextPutAll: aQuantity asString;
		nextPutAll: ' instances of FlowDummyPerson and FlowDummyUser in: ';
		nextPutAll: delay milliseconds asString;
		nextPutAll: ' ms ';
		nextPutAll: '(~',savesPerSecond asString,' saves per second)'
! !

Object subclass: #MaplessMongoReference
	instanceVariableNames: 'behavior oid model'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo'!

!MaplessMongoReference class methodsFor: 'actions' stamp: 'sebastiansastre 12/3/2012 22:42'!
for: aFlowMongoModel

	^ self new
		initializeOn: aFlowMongoModel;
		yourself ! !

!MaplessMongoReference class methodsFor: 'actions' stamp: 'sebastiansastre 12/7/2012 17:44'!
fromJsonObject: aJsonObject

	^ self new
		oid: aJsonObject oid;
		behavior: aJsonObject behavior;
		yourself ! !

!MaplessMongoReference methodsFor: 'accessing' stamp: 'sebastiansastre 12/4/2012 18:55'!
= anObject

	^ super = anObject or:[
	self oid = anObject oid and:[
	self behavior = anObject behavior ] ]! !

!MaplessMongoReference methodsFor: 'accessing' stamp: 'sebastiansastre 12/1/2012 06:51'!
behavior
	^ behavior! !

!MaplessMongoReference methodsFor: 'accessing' stamp: 'sebastiansastre 12/1/2012 06:51'!
behavior: anObject
	behavior := anObject! !

!MaplessMongoReference methodsFor: 'accessing' stamp: 'sebastiansastre 12/4/2012 19:09'!
doesNotUnderstand: aMessage
	"The idea behind this DNU is to use the selector as setters or getter 
	delegating to data (aJsonObject)"
		
	^ self model 
		perform: aMessage 
		selector withArguments: aMessage arguments 	! !

!MaplessMongoReference methodsFor: 'accessing' stamp: 'sebastiansastre 12/3/2012 23:23'!
getMongoModel

	| class |
	
	class := Smalltalk 
				at: behavior asSymbol 
				ifAbsent:[ self error: 'This image does not have ',behavior asString,' class' ].
	
	^ class findAt: oid 
	
	! !

!MaplessMongoReference methodsFor: 'testing' stamp: 'sebastiansastre 12/3/2012 23:18'!
hasModel
 
	^ model notNil! !

!MaplessMongoReference methodsFor: 'initialization' stamp: 'sebastiansastre 12/4/2012 19:10'!
initializeModel 
	"Note this reference will load the model from the database only once,
	usually in the first message send"
	
	model := self getMongoModel.
	model isUnsaved ifFalse:[
		oid := model oid ].
	
	^ model
! !

!MaplessMongoReference methodsFor: 'initialization' stamp: 'sebastiansastre 12/3/2012 22:29'!
initializeOn: aFlowMongoModel

	oid := aFlowMongoModel oid.
	behavior := aFlowMongoModel class name asString.
	
	model := aFlowMongoModel ! !

!MaplessMongoReference methodsFor: 'testing' stamp: 'sebastiansastre 12/5/2012 20:31'!
isKindOf: aClasss

	^ self model isKindOf: aClasss ! !

!MaplessMongoReference methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 19:13'!
jsonWriteOn: aStream

	"The only valid scenario when this won't have an oid 
	is when it has an unsaved model set"
	oid ifNil:[ oid := model oid ].

	"a nil oid at this point should never happen"	
	self assert: oid notNil.

	(JsonObject new
		at: 'behavior' put: behavior;
		at: 'oid' put: oid;
		yourself) jsonWriteOn: aStream! !

!MaplessMongoReference methodsFor: 'accessing' stamp: 'sebastiansastre 12/4/2012 19:09'!
model
	^ model ifNil:[ self initializeModel ]! !

!MaplessMongoReference methodsFor: 'accessing' stamp: 'sebastiansastre 12/4/2012 19:08'!
model: anObject
	model := anObject! !

!MaplessMongoReference methodsFor: 'accessing' stamp: 'sebastiansastre 12/1/2012 06:51'!
oid
	^ oid! !

!MaplessMongoReference methodsFor: 'accessing' stamp: 'sebastiansastre 12/1/2012 06:51'!
oid: anObject
	oid := anObject! !

Object subclass: #MaplessPersistentModel
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Models'!

!MaplessPersistentModel commentStamp: 'sebastiansastre 3/15/2012 21:49' prior: 0!
Flow models are meant to be treated as aggregates.

We don't care here about identity.

Why?

Because the idea is that you keep these objects small and uncomplicated enough to be more efficienty serialized/deserialized in JSON for using it is some sort of noSQL persistence support.!

MaplessPersistentModel subclass: #MaplessMongoModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo'!

!MaplessMongoModel commentStamp: 'sebastiansastre 11/30/2012 13:01' prior: 0!
FlowMongoModel is the abstract class from where subclasses could store and retrieve themselves from and to a MongoDB


!

MaplessMongoModel subclass: #MaplessDummyPerson
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo'!

MaplessMongoModel subclass: #MaplessDummyUser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Mongo'!

!MaplessDummyUser class methodsFor: 'accessing' stamp: 'sebastiansastre 12/4/2012 18:08'!
findUsername: anUsername

	^ self fromJsonObject: ( self databaseCollection detect: [ :each | 
		( each username = anUsername ) ] )! !

!MaplessDummyUser methodsFor: 'accessing' stamp: 'sebastiansastre 12/3/2012 23:29'!
completeName

	^ self person firstName, ' ', self person lastName! !

!MaplessMongoModel class methodsFor: 'accessing' stamp: 'sebastiansastre 11/30/2012 13:30'!
database
	"Answers the current database (or nil)"

	^ CurrentMongoDatabase value! !

!MaplessMongoModel class methodsFor: 'accessing' stamp: 'sebastiansastre 11/30/2012 13:37'!
databaseCollection
	"Answers the collection for this kind of models hold by the current database"
	
	self ensureCollection.
	
	^ self database collectionAt: self databaseCollectionName! !

!MaplessMongoModel class methodsFor: 'accessing' stamp: 'sebastiansastre 11/30/2012 13:30'!
databaseCollectionName
	"Answers the name for the collection for this kind of models"
	
	^ self name asString! !

!MaplessMongoModel class methodsFor: 'actions' stamp: 'sebastiansastre 11/30/2012 13:38'!
ensureCollection
	"Makes sure there is a collection for this kind of models."
	
	(self database hasCollectionNamed: self databaseCollectionName) ifFalse:[
		self database addCollection: self databaseCollectionName]
! !

!MaplessMongoModel class methodsFor: 'testing' stamp: 'sebastiansastre 12/4/2012 14:25'!
existsAt: aMongoOID

	^ ( self findAt: aMongoOID ) notNil! !

!MaplessMongoModel class methodsFor: 'accessing' stamp: 'sebastiansastre 11/30/2012 17:15'!
findAt: aMongoOID

	| oid result |

	oid := Dictionary new 
			at: '_id' put: aMongoOID;
			yourself.

	result := self databaseCollection queryOne:[:q| 
				q where: oid].
			
	^ result isNil
		ifTrue: [ nil ]
		ifFalse: [ self fromJsonObject: result asJsonObject ]
		! !

!MaplessMongoModel methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 11:31'!
asMongoDocument
	"Returns a JsonObject that's BSON friendly."
	
	^ self asMongoDocumentFor: data! !

!MaplessMongoModel methodsFor: 'actions' stamp: 'sebastiansastre 1/4/2013 09:08'!
asMongoDocumentFor: aJsonObject
	"Returns a new JsonObject that's the BSON friendly version of aJsonObject
	by visinting all the values. It will complain if sub models are not previously saved."
	
	| mongoDocument part |
		
	mongoDocument := JsonObject new.
	
	aJsonObject keysAndValuesDo: [ :k :v |
		(v class = MaplessMongoReference)
			ifTrue: [ 
				v isUnsaved ifTrue:[ 
					MaplessUnsavedMongoSubmodel signal: 'This sub model is unsaved. You need to save all sub models before saving a composed model' ]. 
				part := v asJsonString ]
			ifFalse: [ part := v ].
		
		mongoDocument 
			at: k
			put: part ].

	^ mongoDocument ! !

!MaplessMongoModel methodsFor: 'actions' stamp: 'sebastiansastre 11/30/2012 13:32'!
beDeleted

	self isDeleted: true! !

!MaplessMongoModel methodsFor: 'accessing' stamp: 'sebastiansastre 11/30/2012 13:31'!
database 
	"Answers the database that is currently being used."
	
	^ self class database ! !

!MaplessMongoModel methodsFor: 'accessing' stamp: 'sebastiansastre 11/30/2012 13:40'!
databaseCollection 
	"Answers the database collection dedicated to this kind of instances."
	
	^ self class databaseCollection ! !

!MaplessMongoModel methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 14:12'!
delete
	"Saves this instance as deleted in the persistent collection"
	
	self onAboutToDelete.
	
	self beDeleted; save.
	
	self onDeleted 
! !

!MaplessMongoModel methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 14:11'!
destroy
	"Removes this instance from the persistent collection"

	| bsonFriendly |
	
	bsonFriendly := self asMongoDocument.

	self onAboutToDestroy.
		
	self class databaseCollection remove: bsonFriendly.
	
	self onDestroyed ! !

!MaplessMongoModel methodsFor: 'accessing' stamp: 'sebastiansastre 1/4/2013 09:05'!
doesNotUnderstand: aMessage
	"The idea behind this DNU is to use the selector as setters or getter 
	delegating to data (aJsonObject). 
	The mongo model specialization will store sub mongo models wrapped
	in references so serialization and save order is conveniently done."
	
	| key part |

	key := aMessage selector.
	key isUnary ifTrue: [ ^ super doesNotUnderstand: aMessage ].

	^ (key isKeyword and: [(key occurrencesOf: $:) = 1])
		ifTrue: [
			key := key allButLast asSymbol.
			part := aMessage arguments first.
			(part isKindOf: MaplessMongoModel) ifTrue:[
				part := MaplessMongoReference for: part ].
			data at: key asString put: part]
		ifFalse: [super doesNotUnderstand: aMessage]
				
	! !

!MaplessMongoModel methodsFor: 'actions' stamp: 'sebastiansastre 12/3/2012 18:43'!
ensureOID

	super ensureId.
	
	self hasOID ifFalse:[
		data at: '_id' put: self id]! !

!MaplessMongoModel methodsFor: 'testing' stamp: 'sebastiansastre 11/30/2012 16:44'!
hasOID 

	^ self oid notNil! !

!MaplessMongoModel methodsFor: 'initialization' stamp: 'paulwilke 12/8/2012 14:48'!
initializeOn: aJsonObject 
	"Makes this model to be what aJsonObject dictates it to be."

	data := aJsonObject.
	
	self setSubModelProxies
! !

!MaplessMongoModel methodsFor: 'actions' stamp: 'sebastiansastre 12/6/2012 08:15'!
insert
	
	| bsonFriendly |
		
	self onAboutToInsert.

	[bsonFriendly := self asMongoDocument] ifCurtailed:[
		data removeKey: '_id' ifAbsent: [ nil ] ] .

	self class databaseCollection add: bsonFriendly.
	
	self onInserted
! !

!MaplessMongoModel methodsFor: 'testing' stamp: 'sebastiansastre 12/4/2012 13:54'!
isDeleted

	^ super isDeleted notNil and:[
	super isDeleted]! !

!MaplessMongoModel methodsFor: 'testing' stamp: 'sebastiansastre 12/3/2012 18:31'!
isUnsaved
	"Answer true if this instance was not saved yet"
	^ self oid isNil ! !

!MaplessMongoModel methodsFor: 'accessing' stamp: 'sebastiansastre 11/30/2012 16:44'!
oid

	^ data at: '_id'! !

!MaplessMongoModel methodsFor: 'reactions' stamp: 'sebastiansastre 11/30/2012 14:28'!
onAboutToDelete! !

!MaplessMongoModel methodsFor: 'reactions' stamp: 'sebastiansastre 11/30/2012 14:29'!
onAboutToDestroy! !

!MaplessMongoModel methodsFor: 'reactions' stamp: 'sebastiansastre 12/4/2012 10:44'!
onAboutToInsert

	self ensureOID
! !

!MaplessMongoModel methodsFor: 'reactions' stamp: 'sebastiansastre 12/4/2012 00:00'!
onAboutToSave

! !

!MaplessMongoModel methodsFor: 'reactions' stamp: 'sebastiansastre 12/3/2012 19:26'!
onAboutToUpdate! !

!MaplessMongoModel methodsFor: 'reactions' stamp: 'sebastiansastre 11/30/2012 14:29'!
onDeleted! !

!MaplessMongoModel methodsFor: 'reactions' stamp: 'sebastiansastre 11/30/2012 14:29'!
onDestroyed! !

!MaplessMongoModel methodsFor: 'reactions' stamp: 'sebastiansastre 12/3/2012 19:36'!
onInserted ! !

!MaplessMongoModel methodsFor: 'reactions' stamp: 'sebastiansastre 11/30/2012 14:28'!
onSaved! !

!MaplessMongoModel methodsFor: 'reactions' stamp: 'sebastiansastre 12/3/2012 19:36'!
onUpdated! !

!MaplessMongoModel methodsFor: 'actions' stamp: 'sebastiansastre 12/3/2012 19:35'!
save
	"Storage this instance into the persistent collection"
	
	self onAboutToSave.
	
	self isUnsaved
		ifTrue:[ self insert ]
		ifFalse:[ self update ].
		
	self onSaved! !

!MaplessMongoModel methodsFor: 'actions' stamp: 'sebastiansastre 1/4/2013 09:05'!
setSubModelProxies
	"Creates proxies to sub models regarding 
	to the JSON data this instance."

	data keysAndValuesDo: [ :k :v |
		(v isString and:[
		('*oid*' match: v) and:[
		('*behavior*' match: v) ] ] )  ifTrue:[ | jsonObject |
			jsonObject := [ Json readFrom: v readStream ] ifError: [ :x| nil ].
			(jsonObject notNil and:[
			jsonObject oid notNil and:[
			jsonObject behavior notNil]]) ifTrue:[ 
				 data at: k put: ( MaplessMongoReference fromJsonObject: jsonObject ) ] ] ]! !

!MaplessMongoModel methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 11:52'!
update

	| bsonFriendly |
	
	bsonFriendly := self asMongoDocument.

	self onAboutToUpdate.

	self class databaseCollection 
		replace:  (Dictionary new 
					at: '_id' put: self oid;
					yourself)
		with: bsonFriendly.
		
	self onUpdated.! !

!MaplessMongoModel methodsFor: 'actions' stamp: 'sebastiansastre 1/4/2013 09:08'!
validateSavedSubModels
	"Navigates this instance's content in search of unsaved sub models
	and complain about it."
	
	data values do:[ :e |
		(e isKindOf: MaplessMongoReference) ifTrue:[
			e isUnsaved
				ifFalse: [ e validateSavedSubModels ]
				ifTrue:[MaplessUnsavedMongoSubmodel signal: 'This sub model is unsaved. You need to save all sub models before saving a composed model' ] ] ]! !

!MaplessPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 12/3/2012 18:02'!
database

	^ self subclassResponsibility ! !

!MaplessPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 11:30'!
findAll 

	^ self database objectsAt: self! !

!MaplessPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 12/4/2012 18:59'!
findId: anId

	^ self subclassResponsibility ! !

!MaplessPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 23:39'!
fromJson: someJson

	^ self basicNew data: (Json readFrom: someJson readStream)! !

!MaplessPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 12/7/2012 17:45'!
fromJsonObject: aJsonObjectOrNil

	aJsonObjectOrNil ifNil:[ ^ nil ].

	^ self basicNew 
		initializeOn: aJsonObjectOrNil asJsonObject;
		yourself 
! !

!MaplessPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 11/30/2012 12:53'!
fromJsonObjects: someJsonObjects

	^ someJsonObjects collect:[:e|
		self fromJsonObject: e]! !

!MaplessPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 10:00'!
fromReified: aJsonObject 

	^ self fromJsonObject: aJsonObject ! !

!MaplessPersistentModel class methodsFor: 'actions' stamp: 'sebastiansastre 3/4/2012 19:31'!
makeId

	^ UUID new asString36! !

!MaplessPersistentModel methodsFor: 'testing' stamp: 'sebastiansastre 3/17/2012 14:43'!
= aModel 
	"Answers true if this model is the same as aModel.
	Note: as you can see, this won't care about identity (yes, that's intentional)."
	^ self id = aModel id! !

!MaplessPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 11/27/2012 22:44'!
asBSON
	
	^ BSON write: self asBSONFriendly ! !

!MaplessPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 11/27/2012 22:44'!
asBSONFriendly

	^ Dictionary new 
		at: 'key' put: self id;
		at: 'value' put: self asJsonString;
		yourself! !

!MaplessPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 11/30/2012 13:43'!
asDictionary 
	"Answers this instance as a JSON friendly dictionary"
	
	^ [:dic| | attributes |
		attributes := OrderedJsonObject new.
		dic at: self class name asString put: attributes.
		self data associationsDo:[:assoc|
			assoc value isPersistentModel 
				ifTrue:[ attributes at: assoc key put: assoc value asDictionary ]
				ifFalse:[ attributes at: assoc key put: assoc value] ].
		dic ] value: OrderedJsonObject new! !

!MaplessPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 11:58'!
asJsonString
	
	self onAboutToJSON.
	
	^ super asJsonString! !

!MaplessPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 10:43'!
at: aKey

	^ data at: aKey! !

!MaplessPersistentModel methodsFor: 'actions' stamp: 'sebastiansastre 3/24/2012 10:43'!
at: aKey put: anObject

	^ data at: aKey put: anObject! !

!MaplessPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 11:43'!
createdOn 

	| selector expects object|

	selector := #createdOn.
	expects := DateAndTime.

	object := self at: selector asString.
	object ifNil:[^nil].

	(object isKindOf: expects) ifTrue:[^object].

	^ self at: selector asString put: (self dateAndTimeAt: selector).
! !

!MaplessPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 23:32'!
data

	^ data! !

!MaplessPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 23:32'!
data: aJsonObject

	data := aJsonObject 
	! !

!MaplessPersistentModel methodsFor: 'actions' stamp: 'sebastiansastre 3/24/2012 11:47'!
dateAndTimeAt: aSelector

	^ DateAndTime fromString: (self at: aSelector)! !

!MaplessPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/19/2012 21:56'!
doesNotUnderstand: aMessage
	"The idea behind this DNU is to use the selector as setters or getter 
	delegating to data (aJsonObject)"
	
	| key |
	key := aMessage selector.
	key isUnary ifTrue: [^ data at: key asString].
	^ (key isKeyword and: [(key occurrencesOf: $:) = 1])
		ifTrue: [key := key allButLast asSymbol.
				data at: key asString put: aMessage arguments first]
		ifFalse: [super doesNotUnderstand: aMessage]
				
	! !

!MaplessPersistentModel methodsFor: 'actions' stamp: 'sebastiansastre 11/30/2012 13:45'!
ensureId 

	self id isNil ifTrue:[
		self id: self class makeId].! !

!MaplessPersistentModel methodsFor: 'actions' stamp: 'sebastiansastre 12/4/2012 19:00'!
fresh

	^ self isUnsaved 
		ifTrue: [ self ]
		ifFalse: [ self class findId: self id ]! !

!MaplessPersistentModel methodsFor: 'initialization' stamp: 'sebastiansastre 11/30/2012 13:44'!
initialize

	super initialize.
	
	data := OrderedJsonObject new.
	
	self ensureId.
	
	"We include the class name so we have the minimal metadata at consuming time"
	self class: self class name.
	self createdOn: DateAndTime now.
! !

!MaplessPersistentModel methodsFor: 'testing' stamp: 'sebastiansastre 3/27/2012 10:43'!
isPersistentModel

	^ true! !

!MaplessPersistentModel methodsFor: 'testing' stamp: 'sebastiansastre 12/4/2012 19:00'!
isUnsaved
	"Answers true if this instance is not saved yet"
	^ self subclassResponsibility ! !

!MaplessPersistentModel methodsFor: 'actions' stamp: 'sebastiansastre 3/18/2012 23:53'!
jsonWriteOn: aStream

	data jsonWriteOn: aStream! !

!MaplessPersistentModel methodsFor: 'reactions' stamp: 'sebastiansastre 1/4/2013 09:15'!
onAboutToJSON
	"This model is about to be stringified in JSON.
	All inst var values that are expected to be model objects, need to be instanciated as expected."

	data keys do:[:key| |value|
		value := data at: key.
		"If the value is a js object proxy, we assume it's the data for a (composed) flow model, 
		so we call the getter in this receiver expecting to get it properly instantiated (because that's our convention here)."
		(value isKindOf: JsonObject) ifTrue:[
			(self respondsTo: key asSymbol) ifFalse:[MaplessModelGetterIsMissing signal: 'Cannot find getter for: ', key].
			value := self perform: key asSymbol.
			value onAboutToJSON.
			self at: key put: value]].
	! !

!MaplessPersistentModel methodsFor: 'reactions' stamp: 'sebastiansastre 3/27/2012 10:48'!
onRead
	"This model has been readed.
	All inst var values that are expected to be model objects, need to be re-instanciated."

	data keys do:[:key| |value|
		value := data at: key.
		value isPersistentModel ifTrue:[
			value := value fresh.
			self at: key put: value]].
	! !

!MaplessPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 1/4/2013 09:15'!
subModelAt: aSelector
	"Answers the reified submodel (instantiating if necessary)."
	
	| subModelData class |

	subModelData := data at: aSelector.
	subModelData ifNil:[^nil].

	"legacy code from Amber's json serialization
	subModelData := subModelData at: '@data'.
	subModelData ifNil:[^ FlowModelDataError signal: 'Cannot find ', aSelector,'''s data']."
	
	class := subModelData at: 'class'.
	class := Smalltalk at: class asSymbol ifAbsent:[
		MaplessModelMetadataError signal: 'Cannot find ',aSelector,'''s metadata'].
	
	^ class fromReified: subModelData	! !

MaplessPersistentModel subclass: #ToDo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Models'!

MaplessPersistentModel subclass: #ToDoList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mapless-Models'!
